"""
This module contains ChannelObject implementation.

There is no need to import this module directly. Use the methods of
the L{Ruleset object <ruleset.Ruleset>} to create and manipulate
ChannelObjects.

    >>> o = r.object_add("/ChannelObjects/Test")
    >>> o.add("101")
    >>> print o.items
    ['101']

@var __revision__: Revision identifier.
"""

__revision__ = '%K%'

import plobject
from thisversion import PLDB
import plutil
from plutil import verify_ip

def _verify_channel(channel_id):
	if channel_id < 0 or channel_id > (1 << 16):
		return False
	return True

OBJECT_TYPE = -1

class ChannelObject(plobject.PLObject):
	"""
	ChannelObject Implementation

	Object path: /ChannelObjects

	@cvar Item: ChannelObject Item implementation.
	"""

	__slots__ = ('_prop', '_pldb', '_callback', '_item_added')
	_may_exclude = True

	base_path = plutil.roConst("/ChannelObjects", "Only the type portion of the path")
	type = plutil.roConst(OBJECT_TYPE, "Number representing the type of object")

	def _create_item(self, value):
		item = self.Item(self, type=PLDB.CHANNELOBJECT_ITEM_TYPE_CHANNEL, value=value)

		if not _verify_channel(item._prop.value):
			raise ValueError("First argument doesn't seem like a Channel id")

		return item

	def add(self, value, exclude=False):
		"""
		Adds a ChannelObject item to this ChannelObject.

		@type value: C{str}
		@param value: The value is a string describing the ChannelObject item. The type of the item
			will be automatically detected from the string. The following syntax is valid:
				- Channel ID, just give the plain Channel id I{XXXX}

		@type exclude: C{boolean}
		@param exclude: If this is set to True the item will be a exclude
			Item instead of a include Item.

		@rtype: C{None}
		"""
		plobject.PLObject.add(self, value, exclude)


	class Item(plobject.PLObject.Item):
		"""
		ChannelObject Item implementation.
		Should not be instantiated, use L{ChannelObject.add} instead.

		@type exclude: C{boolean}
		@type value1: C{int}
		@type value2: C{int}
		"""
		__slots__ = ('_prop', '_parent')
		_Prop = plutil.Prop(exclude=bool, type=int, value=int, *plobject._item_props)

		def __str__(self):
			if self._prop.exclude:
				ex = "!"
			else:
				ex = ""

			if self._prop.type == PLDB.CHANNELOBJECT_ITEM_TYPE_CHANNEL:
				return ex + str(self._prop.value)
			else:
				raise ValueError("Wrong item-type for this object")

		type = property(lambda self: self._prop.type, doc="Internal number representing the item type (single or range)")
		exclude = property(lambda self: self._prop.exclude, doc="True = exclude, False = include")
		value = property(lambda self: self._prop.value, doc="Channel ID")
